﻿Imports System.Threading
Imports LibUsbDotNet
Imports LibUsbDotNet.Main

Namespace USB

    Friend Module SharedCommon
        Friend Const DEFAULT_TIMEOUT As Integer = 5000

        Friend Const USB_VID_ATMEL As UInt16 = &H3EB
        Friend Const USB_PID_AT90USB162 As UInt16 = &H2FFA 'FCUSB PCB 1.x
        Friend Const USB_PID_AT90USB1287 As UInt16 = &H2FFB 'FCUSB EX (PROTO)
        Friend Const USB_PID_AT90USB646 As UInt16 = &H2FF9 'FCUSB EX (PRODUCTION)
        Friend Const USB_PID_ATMEGA32U2 As UInt16 = &H2FF0 'FCUSB PCB 2.1-2.2
        Friend Const USB_PID_ATMEGA32U4 As UInt16 = &H2FF4 'FCUSB PCB 2.3

        Friend Function GetDevicePath(device As UsbRegistry) As String
            Try
                Dim dev_loc As String = "USB\VID_" & Hex(device.Vid).PadLeft(4, "0") & "&PID_" & Hex(device.Pid).PadLeft(4, "0")
                If device.GetType Is GetType(LibUsb.LibUsbRegistry) Then
                    dev_loc &= "\" & device.DeviceProperties("LocationInformation")
                ElseIf device.GetType Is GetType(LegacyUsbRegistry) Then
                    Dim legacy As LegacyUsbRegistry = DirectCast(device, LegacyUsbRegistry)
                    Dim DeviceFilename As String = DirectCast(legacy.Device, LibUsb.LibUsbDevice).DeviceFilename
                    dev_loc &= DeviceFilename
                End If
                Return dev_loc
            Catch ex As Exception
                Return ""
            End Try
        End Function

    End Module

    Public Class HostClient
        Public Event DeviceConnected(usb_dev As FCUSB_DEVICE)
        Public Event DeviceDisconnected(usb_dev As FCUSB_DEVICE)

        Private ConnectedDevices As New List(Of FCUSB_DEVICE) 'Contains all devices that are connected

        Public Property CloseService As Boolean = False

        Sub New()

        End Sub

        Public Sub StartService()
            Try
                Dim td As New Thread(AddressOf ConnectionThread)
                td.Name = "tdUsbMonitor"
                td.Start()
            Catch ex As Exception
            End Try
        End Sub

        Private Sub ConnectionThread()
            Do While (Not CloseService)
                CheckConnectedDevices()
                If Not CloseService Then ConnectToDevices()
                Thread.Sleep(150)
            Loop
            DisconnectAll()
        End Sub

        Private Sub ConnectToDevices()
            Try
                Dim fcusb_list() As UsbRegistry = FindUsbDevices() 'Returns a list of all FCUSB devices connected
                If fcusb_list IsNot Nothing AndAlso fcusb_list.Length > 0 Then
                    For Each this_fcusb In fcusb_list
                        Dim fcusb_path As String = GetDevicePath(this_fcusb)
                        If Not DevicePathIsConnected(fcusb_path) Then 'This device is not in our list
                            Dim this_dev As UsbDevice = this_fcusb.Device
                            If this_dev Is Nothing Then Exit For
                            Dim new_fcusb_dev As FCUSB_DEVICE = Nothing
                            If Connect(this_dev, new_fcusb_dev) Then
                                ConnectedDevices.Add(new_fcusb_dev)
                                AddHandler new_fcusb_dev.OnDisconnected, AddressOf FCUSB_Device_Disconnected
                                RaiseEvent DeviceConnected(new_fcusb_dev)
                            End If
                        End If
                    Next
                End If
            Catch ex As Exception
            End Try
        End Sub

        Private Sub CheckConnectedDevices()
            Dim disconnected_dev As New List(Of FCUSB_DEVICE)
            For Each dev In ConnectedDevices
                If dev.IS_CONNECTED Then
                    Dim is_online As Boolean = dev.CheckConnection()
                    If Not is_online Then disconnected_dev.Add(dev)
                End If
            Next
            For Each dev In disconnected_dev
                dev.Disconnect()
            Next
        End Sub

        Private Sub FCUSB_Device_Disconnected(disconnected_device As FCUSB_DEVICE)
            Try
                ConnectedDevices.Remove(disconnected_device)
            Catch ex As Exception
            End Try
            RaiseEvent DeviceDisconnected(disconnected_device)
        End Sub

        Private Function DevicePathIsConnected(path As String) As Boolean
            Try
                For Each dev In ConnectedDevices
                    If dev.USB_PATH.Equals(path) Then Return True
                Next
            Catch ex As Exception
            End Try
            Return False 'Not found
        End Function

        Public Function Connect(usb_device As UsbDevice, ByRef fcusb_dev As FCUSB_DEVICE) As Boolean
            Try
                If OpenUsbDevice(usb_device) Then
                    Dim new_device As New FCUSB_DEVICE(usb_device)
                    If (usb_device.UsbRegistryInfo.Vid = USB_VID_ATMEL) Then 'DFU Mode
                        new_device.IS_CONNECTED = True
                        new_device.HWBOARD = FCUSB_BOARD.ATMEL_DFU
                        fcusb_dev = new_device
                        Return True
                    End If
                    Return False
                End If
            Catch ex As Exception
            End Try
            Return False
        End Function

        Private Function OpenUsbDevice(usb_dev As UsbDevice) As Boolean
            Try
                Dim wholeUsbDevice As IUsbDevice = TryCast(usb_dev, IUsbDevice)
                If wholeUsbDevice IsNot Nothing Then 'Libusb only
                    wholeUsbDevice.SetConfiguration(1)
                    wholeUsbDevice.ClaimInterface(0)
                    Try
                        wholeUsbDevice.SetAltInterface(1)
                    Catch ex As Exception
                    End Try
                End If
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Private Function FindUsbDevices() As UsbRegistry()
            Try
                Dim devices As New List(Of UsbRegistry)
                AddDevicesToList(USB_VID_ATMEL, USB_PID_AT90USB162, devices)
                AddDevicesToList(USB_VID_ATMEL, USB_PID_AT90USB1287, devices)
                AddDevicesToList(USB_VID_ATMEL, USB_PID_ATMEGA32U2, devices)
                AddDevicesToList(USB_VID_ATMEL, USB_PID_AT90USB646, devices)
                AddDevicesToList(USB_VID_ATMEL, USB_PID_ATMEGA32U4, devices)
                If devices.Count = 0 Then Return Nothing
                Return devices.ToArray
            Catch ex As Exception
            End Try
            Return Nothing
        End Function

        Private Sub AddDevicesToList(VID As UInt16, PID As UInt16, DeviceList As List(Of UsbRegistry))
            Dim fcusb_usb_device As New UsbDeviceFinder(VID, PID)
            Dim fcusb_list As UsbRegDeviceList = UsbDevice.AllDevices.FindAll(fcusb_usb_device)
            If fcusb_list IsNot Nothing AndAlso (fcusb_list.Count > 0) Then
                For i = 0 To fcusb_list.Count - 1
                    If fcusb_list(i).GetType IsNot GetType(WinUsb.WinUsbRegistry) Then
                        DeviceList.Add(fcusb_list(i))
                    End If
                Next
            End If
        End Sub

        Public Sub DisconnectAll()
            Try
                Dim dev_list As New List(Of FCUSB_DEVICE)
                For Each fc_devs In Me.ConnectedDevices
                    dev_list.Add(fc_devs)
                Next
                For Each fc_devs In dev_list
                    fc_devs.Disconnect()
                Next
            Catch ex As Exception
            End Try
        End Sub

    End Class

    Public Class FCUSB_DEVICE
        Public USBHANDLE As UsbDevice = Nothing
        Public Property HWBOARD As FCUSB_BOARD
        Public ReadOnly Property USB_PATH As String
        Public Property IS_CONNECTED As Boolean = False
        Public Property USBFLAG_OUT As UsbCtrlFlags
        Public Property USBFLAG_IN As UsbCtrlFlags

        Public DFU_IF As New DFU_Programmer(Me)

        Private USB_TIMEOUT_VALUE As Integer = DEFAULT_TIMEOUT

        Public Event OnDisconnected(this_dev As FCUSB_DEVICE)
        Public Event OnUpdateProgress(device As FCUSB_DEVICE, percent As Integer)
        Public Event OnPrintConsole(device As FCUSB_DEVICE, msg As String)

        Sub New(my_handle As UsbDevice)
            Me.USBHANDLE = my_handle
            Me.USB_PATH = GetDevicePath(my_handle.UsbRegistryInfo)
            AddHandler DFU_IF.PrintConsole, AddressOf PrintConsole
            Me.USBFLAG_OUT = (UsbCtrlFlags.RequestType_Vendor Or UsbCtrlFlags.Recipient_Device Or UsbCtrlFlags.Direction_Out)
            Me.USBFLAG_IN = (UsbCtrlFlags.RequestType_Vendor Or UsbCtrlFlags.Recipient_Device Or UsbCtrlFlags.Direction_In)
        End Sub

        Public Function CheckConnection() As Boolean
            If USBHANDLE Is Nothing Then Return False
            If Not USBHANDLE.UsbRegistryInfo.IsAlive Then Return False
            Return True
        End Function

        Public Sub Disconnect()
            Try
                If Not Me.IS_CONNECTED Then Exit Sub
                Me.IS_CONNECTED = False
                If Me.USBHANDLE IsNot Nothing Then
                    Dim wholeUsbDevice As IUsbDevice = TryCast(Me.USBHANDLE, IUsbDevice)
                    If wholeUsbDevice IsNot Nothing Then 'Libusb only
                        wholeUsbDevice.ReleaseInterface(0)
                    End If
                    Me.USBHANDLE.Close()
                    Me.USBHANDLE = Nothing
                End If
            Catch ex As Exception
            End Try
            RaiseEvent OnDisconnected(Me)
        End Sub

        Private Sub PrintConsole(msg As String)
            RaiseEvent OnPrintConsole(Me, msg)
        End Sub

#Region "USB Control"

        Public Function USB_SETUP_BULKOUT(RQ As USBREQ, SETUP() As Byte, BULK_OUT() As Byte, control_dt As UInt32, Optional timeout As Integer = -1) As Boolean
            Try
                Dim result As Boolean = True = USB_CONTROL_MSG_OUT(RQ, SETUP, control_dt) 'Sends setup command and data
                If Not result Then Return False
                If BULK_OUT Is Nothing Then Return True
                result = USB_BULK_OUT(BULK_OUT)
                Return result
            Catch ex As Exception
                Return False
            End Try
        End Function

        Public Function USB_SETUP_BULKIN(RQ As USBREQ, SETUP() As Byte, ByRef DATA_IN() As Byte, control_dt As UInt32, Optional timeout As Integer = -1) As Boolean
            Try
                Dim result As Boolean = USB_CONTROL_MSG_OUT(RQ, SETUP, control_dt) 'Sends setup command and data
                If Not result Then Return False
                result = USB_BULK_IN(DATA_IN, timeout)
                Return result
            Catch ex As Exception
                Return False
            End Try
        End Function
        'Sends a control message with an optional byte buffer to write
        Public Function USB_CONTROL_MSG_OUT(RQ As USBREQ, Optional buffer_out() As Byte = Nothing, Optional data As UInt32 = 0) As Boolean
            Try
                If USBHANDLE Is Nothing Then Return False
                Dim result As Boolean
                Dim wValue As UInt16 = (data And &HFFFF0000UI) >> 16
                Dim wIndex As UInt16 = (data And &HFFFF)
                Dim bytes_out As Integer = 0
                If buffer_out IsNot Nothing Then bytes_out = buffer_out.Length
                Dim usbSetupPacket As New UsbSetupPacket(Me.USBFLAG_OUT, RQ, wValue, wIndex, CShort(bytes_out))
                Dim bytes_xfer As Integer = 0
                If buffer_out Is Nothing Then
                    result = USBHANDLE.ControlTransfer(usbSetupPacket, Nothing, 0, bytes_xfer)
                Else
                    result = USBHANDLE.ControlTransfer(usbSetupPacket, buffer_out, buffer_out.Length, bytes_xfer)
                End If
                Utilities.Sleep(5)
                Return result
            Catch ex As Exception
                Return False
            End Try
        End Function
        'Sends a control message with a byte buffer to receive data
        Public Function USB_CONTROL_MSG_IN(RQ As USBREQ, ByRef Buffer_in() As Byte, Optional data As UInt32 = 0) As Boolean
            Try
                If USBHANDLE Is Nothing Then Return False
                Dim bytes_xfer As Integer = 0
                Dim wValue As UInt16 = (data And &HFFFF0000UI) >> 16
                Dim wIndex As UInt16 = (data And &HFFFF)
                Dim usb_setup As New UsbSetupPacket(Me.USBFLAG_IN, RQ, wValue, wIndex, CShort(Buffer_in.Length))
                Dim result As Boolean = USBHANDLE.ControlTransfer(usb_setup, Buffer_in, Buffer_in.Length, bytes_xfer)
                If Not result Then Return False
                If Not Buffer_in.Length = bytes_xfer Then Return False
                Return True 'No error
            Catch ex As Exception
                Return False
            End Try
        End Function

        Public Function USB_BULK_IN(ByRef buffer_in() As Byte, Optional Timeout As Integer = -1) As Boolean
            Try
                If Timeout = -1 Then Timeout = USB_TIMEOUT_VALUE
                Dim BytesRead As Integer = 0
                Dim reader As UsbEndpointReader = USBHANDLE.OpenEndpointReader(ReadEndpointID.Ep01, buffer_in.Length, EndpointType.Bulk)
                Dim ec As ErrorCode = reader.Read(buffer_in, 0, buffer_in.Length, Timeout, BytesRead)
                If ec = ErrorCode.None Then Return True
            Catch ex As Exception
            End Try
            Return False
        End Function

        Public Function USB_BULK_OUT(buffer_out() As Byte, Optional Timeout As Integer = -1) As Boolean
            Try
                If Timeout = -1 Then Timeout = USB_TIMEOUT_VALUE
                Dim BytesWritten As Integer = 0
                Dim writer As UsbEndpointWriter = USBHANDLE.OpenEndpointWriter(WriteEndpointID.Ep02, EndpointType.Bulk)
                Dim ec As ErrorCode = writer.Write(buffer_out, 0, buffer_out.Length, Timeout, BytesWritten)
                If Not ec = ErrorCode.None Or Not BytesWritten = buffer_out.Length Then Return False
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Public Function USB_WaitForComplete() As Boolean
            Dim timeout_counter As Integer
            Dim task_id As Byte = 255
            Do
                Dim packet_out(0) As Byte
                Utilities.Sleep(5) 'Prevents slamming the USB port
                Dim result As Boolean = USB_CONTROL_MSG_IN(USBREQ.GET_TASK, packet_out)
                If Not result Then
                    task_id = 1 'WAITING
                Else
                    task_id = packet_out(0)
                End If
                timeout_counter += 1
                If (timeout_counter = 1000) Then Return False 'Changed from 500
            Loop While (task_id > 0)
            Return True
        End Function

#End Region

    End Class
    'USB commands
    Public Enum USBREQ As Byte
        JTAG_DETECT = &H10
        JTAG_RESET = &H11
        JTAG_SELECT = &H12
        JTAG_READ_B = &H13
        JTAG_READ_H = &H14
        JTAG_READ_W = &H15
        JTAG_WRITE = &H16
        JTAG_READMEM = &H17
        JTAG_WRITEMEM = &H18
        JTAG_ARM_INIT = &H19
        JTAG_INIT = &H1A
        JTAG_FLASHSPI_BRCM = &H1B
        JTAG_FLASHSPI_ATH = &H1C
        JTAG_FLASHWRITE_I16 = &H1D
        JTAG_FLASHWRITE_A16 = &H1E
        JTAG_FLASHWRITE_SST = &H1F
        JTAG_FLASHWRITE_AMDNB = &H20
        JTAG_SCAN = &H21
        JTAG_TOGGLE = &H22 'Toggle in HIGH values into TDI
        JTAG_GOTO_STATE = &H23
        JTAG_SET_OPTION = &H24
        JTAG_REGISTERS = &H25
        JTAG_SHIFT_DATA = &H27
        JTAG_BDR_SETUP = &H28
        JTAG_BDR_INIT = &H29
        JTAG_BDR_ADDPIN = &H2A
        JTAG_BDR_WRCMD = &H2B
        JTAG_BDR_WRMEM = &H2C
        JTAG_BDR_RDMEM = &H2D
        JTAG_BDR_RDFLASH = &H2E
        JTAG_BDR_WRFLASH = &H2F
        JTAG_BDR_SETBSR = &H30
        JTAG_BDR_WRITEBSR = &H31
        JTAG_SHIFT_IR = &H32
        JTAG_SHIFT_DR = &H33
        SPI_INIT = &H40
        SPI_SS_ENABLE = &H41
        SPI_SS_DISABLE = &H42
        SPI_PROG = &H43
        SPI_RD_DATA = &H44
        SPI_WR_DATA = &H45
        SPI_READFLASH = &H46
        SPI_WRITEFLASH = &H47
        SPI_WRITEDATA_AAI = &H48
        S93_INIT = &H49
        S93_READEEPROM = &H4A
        S93_WRITEEEPROM = &H4B
        S93_ERASE = &H4C
        SQI_SETUP = &H50
        SQI_SS_ENABLE = &H51
        SQI_SS_DISABLE = &H52
        SQI_RD_DATA = &H53
        SQI_WR_DATA = &H54
        SQI_RD_FLASH = &H55
        SQI_WR_FLASH = &H56
        SPINAND_READFLASH = &H5B
        SPINAND_WRITEFLASH = &H5C
        I2C_INIT = &H60
        I2C_READEEPROM = &H61
        I2C_WRITEEEPROM = &H62
        I2C_RESULT = &H63
        EXPIO_INIT = &H64
        EXPIO_ADDRESS = &H65
        EXPIO_WRITEDATA = &H66
        EXPIO_READDATA = &H67
        EXPIO_RDID = &H68
        EXPIO_CHIPERASE = &H69
        EXPIO_SECTORERASE = &H6A
        EXPIO_WRITEPAGE = &H6B
        EXPIO_ADDRESS_CE = &H6F
        EXPIO_MODE_READ = &H73
        EXPIO_MODE_WRITE = &H74
        EXPIO_MODE_DELAY = &H75
        EXPIO_CTRL = &H76
        EXPIO_DELAY = &H78
        EXPIO_WRCMDDATA = &H7A
        EXPIO_WRMEMDATA = &H7B
        EXPIO_RDMEMDATA = &H7C
        EXPIO_WAIT = &H7D
        EXPIO_CPEN = &H7E
        EXPIO_SR = &H7F
        EXPIO_TIMING = &H5E
        VERSION = &H80
        ECHO = &H81
        LEDON = &H82
        LEDOFF = &H83
        LEDBLINK = &H84
        START_SENDING_FIRM = &H85
        SEND_FIRM_SIZE = &H86
        SEND_FIRM_DATA = &H87
        STOP_SEND_FIRM_DATA = &H88
        ABORT = &H89
        VCC_1V8 = &H8A
        VCC_3V = &H8B
        VCC_5V = &H8C
        VCC_ON = &H8D
        VCC_OFF = &H8E
        GET_TASK = &H8F
        LOAD_PAYLOAD = &H90
        READ_PAYLOAD = &H91
        FW_UPDATE = &H94 'Update the firmware/bootloader
        FW_REBOOT = &H97
        TEST_READ = &HA1
        TEST_WRITE = &HA2
        SWI_DETECT = &HB0
        SWI_READ = &HB1
        SWI_WRITE = &HB2
        SWI_RD_REG = &HB3
        SWI_WR_REG = &HB4
        SWI_LOCK_REG = &HB5
        PULSE_RESET = &HB6
        LOGIC_STATUS = &HC0
        LOGIC_OFF = &HC1  'Turns off LOGIC circuit
        LOGIC_1V8 = &HC2  'Turns on 1.8V and then LOGIC
        LOGIC_3V3 = &HC3  'Turns on 3.3V and then LOGIC
        LOGIC_VERSION_GET = &HC4  'returns the LOGIC version from the Flash
        LOGIC_VERSION_SET = &HC5  'Writes the LOGIC to the Flash
        SPI_REPEAT = &HC6
        EPROM_RESULT = &HC7
        LOGIC_START = &HC8
        NAND_ONFI = &HD0
        NAND_SR = &HD1
        NAND_SETTYPE = &HD2
        JUMP_BOOT = &HF4
    End Enum

    Public Enum FCUSB_BOARD
        NotSupported 'PCB4.0/XPORT 1.x
        ATMEL_DFU 'Bootloader (either ATMEL_DFU or FC_BL)
        Classic 'Classic (PCB 2.x)
        XPORT_PCB2 'XPORT firmware (PCB 2.x)
        Professional_PCB5
        Mach1 '(PCB 2.x)
    End Enum

End Namespace